@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtDBus module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -42,6 +42,7 @@
 #include <qdebug.h>
 #include <qcoreapplication.h>
 #include <qstringlist.h>
+#include <qthread.h>
 
 #include "qdbusconnection.h"
 #include "qdbusconnectioninterface.h"
@@ -51,6 +52,7 @@
 #include "qdbusconnection_p.h"
 #include "qdbusinterface_p.h"
 #include "qdbusutil_p.h"
+#include "qdbusconnectionmanager_p.h"
 
 #include "qdbusthreaddebug_p.h"
 
@@ -58,27 +60,6 @@
 
 QT_BEGIN_NAMESPACE
 
-class QDBusConnectionManager
-{
-public:
-    QDBusConnectionManager() {}
-    ~QDBusConnectionManager();
-
-    QDBusConnectionPrivate *connection(const QString &name) const;
-    void removeConnection(const QString &name);
-    void setConnection(const QString &name, QDBusConnectionPrivate *c);
-
-    QDBusConnectionPrivate *sender() const;
-    void setSender(const QDBusConnectionPrivate *s);
-
-    mutable QMutex mutex;
-private:
-    QHash<QString, QDBusConnectionPrivate *> connectionHash;
-
-    mutable QMutex senderMutex;
-    QString senderName; // internal; will probably change
-};
-
 Q_GLOBAL_STATIC(QDBusConnectionManager, _q_manager)
 
 QDBusConnectionPrivate *QDBusConnectionManager::sender() const
@@ -125,6 +106,11 @@ QDBusConnectionManager::~QDBusConnectionManager()
     connectionHash.clear();
 }
 
+QDBusConnectionManager* QDBusConnectionManager::instance()
+{
+    return _q_manager();
+}
+
 Q_DBUS_EXPORT void qDBusBindToApplication();
 void qDBusBindToApplication()
 {
@@ -140,9 +126,9 @@ void QDBusConnectionManager::setConnection(const QString &name, QDBusConnectionP
     \fn QDBusConnection &QDBusConnection::sessionBus()
     \relates QDBusConnection
 
-    Returns a QDBusConnection object opened with the session bus. The object reference returned
-    by this function is valid until the QCoreApplication's destructor is run, when the
-    connection will be closed and the object, deleted.
+    Returns a QDBusConnection object opened with the session bus. The object
+    reference returned by this function is valid until the application terminates,
+    at which point the connection will be closed and the object deleted.
 */
 /*!
     \fn QDBusConnection &QDBusConnection::systemBus()
@@ -235,6 +221,18 @@ void QDBusConnectionManager::setConnection(const QString &name, QDBusConnectionP
     \sa registerObject(), QDBusAbstractAdaptor, {usingadaptors.html}{Using adaptors}
 */
 
+/*!
+    \internal
+    \since 4.8
+    \enum QDBusConnection::VirtualObjectRegisterOption
+    Specifies the options for registering virtual objects with the connection. The possible values are:
+
+    \value SingleNode                           register a virtual object to handle one path only
+    \value SubPath                              register a virtual object so that it handles all sub paths
+
+    \sa registerVirtualObject(), QDBusVirtualObject
+*/
+
 /*!
     \enum QDBusConnection::UnregisterMode
     The mode for unregistering an object path:
@@ -246,6 +244,18 @@ void QDBusConnectionManager::setConnection(const QString &name, QDBusConnectionP
     will unregister the child objects too.
 */
 
+/*!
+    \since 4.8
+    \enum QDBusConnection::ConnectionCapability
+
+    This enum describes the available capabilities for a D-Bus connection.
+
+    \value UnixFileDescriptorPassing        enables passing of Unix file descriptors to other processes
+                                            (see QDBusUnixFileDescriptor)
+
+    \sa connectionCapabilities()
+*/
+
 /*!
     Creates a QDBusConnection object attached to the connection with name \a name.
 
@@ -359,7 +369,7 @@ QDBusConnection QDBusConnection::connectToBus(BusType type, const QString &name)
 }
 
 /*!
-    Opens a peer-to-peer connection on address \a address and associate with it the
+    Opens a connection to a private bus on address \a address and associate with it the
     connection name \a name. Returns a QDBusConnection object associated with that connection.
 */
 QDBusConnection QDBusConnection::connectToBus(const QString &address,
@@ -367,7 +377,7 @@ QDBusConnection QDBusConnection::connectToBus(const QString &address,
 {
 //    Q_ASSERT_X(QCoreApplication::instance(), "QDBusConnection::addConnection",
 //               "Cannot create connection without a Q[Core]Application instance");
-    if (!qdbus_loadLibDBus()){
+    if (!qdbus_loadLibDBus()) {
         QDBusConnectionPrivate *d = 0;
         return QDBusConnection(d);
     }
@@ -399,9 +409,43 @@ QDBusConnection QDBusConnection::connectToBus(const QString &address,
 
     return retval;
 }
+/*!
+    \since 4.8
+
+    Opens a peer-to-peer connection on address \a address and associate with it the
+    connection name \a name. Returns a QDBusConnection object associated with that connection.
+*/
+QDBusConnection QDBusConnection::connectToPeer(const QString &address,
+                                               const QString &name)
+{
+//    Q_ASSERT_X(QCoreApplication::instance(), "QDBusConnection::addConnection",
+//               "Cannot create connection without a Q[Core]Application instance");
+    if (!qdbus_loadLibDBus()) {
+        QDBusConnectionPrivate *d = 0;
+        return QDBusConnection(d);
+    }
+
+    QMutexLocker locker(&_q_manager()->mutex);
+
+    QDBusConnectionPrivate *d = _q_manager()->connection(name);
+    if (d || name.isEmpty())
+        return QDBusConnection(d);
+
+    d = new QDBusConnectionPrivate;
+    // setPeer does the error handling for us
+    QDBusErrorInternal error;
+    DBusConnection *c = q_dbus_connection_open_private(address.toUtf8().constData(), error);
+
+    d->setPeer(c, error);
+    _q_manager()->setConnection(name, d);
+
+    QDBusConnection retval(d);
+
+    return retval;
+}
 
 /*!
-    Closes the connection of name \a name.
+    Closes the bus connection of name \a name.
 
     Note that if there are still QDBusConnection objects associated
     with the same connection, the connection will not be closed until
@@ -412,6 +456,30 @@ void QDBusConnection::disconnectFromBus(const QString &name)
 {
     if (_q_manager()) {
         QMutexLocker locker(&_q_manager()->mutex);
+        QDBusConnectionPrivate *d = _q_manager()->connection(name);
+        if (d && d->mode != QDBusConnectionPrivate::ClientMode)
+            return;
+        _q_manager()->removeConnection(name);
+    }
+}
+
+/*!
+    \since 4.8
+
+    Closes the peer connection of name \a name.
+
+    Note that if there are still QDBusConnection objects associated
+    with the same connection, the connection will not be closed until
+    all references are dropped. However, no further references can be
+    created using the QDBusConnection constructor.
+*/
+void QDBusConnection::disconnectFromPeer(const QString &name)
+{
+    if (_q_manager()) {
+        QMutexLocker locker(&_q_manager()->mutex);
+        QDBusConnectionPrivate *d = _q_manager()->connection(name);
+        if (d && d->mode != QDBusConnectionPrivate::PeerMode)
+            return;
         _q_manager()->removeConnection(name);
     }
 }
@@ -632,10 +700,26 @@ bool QDBusConnection::connect(const QString &service, const QString &path, const
 
     if (!receiver || !slot || !d || !d->connection)
         return false;
-    if (!interface.isEmpty() && !QDBusUtil::isValidInterfaceName(interface))
-        return false;
     if (interface.isEmpty() && name.isEmpty())
         return false;
+    if (!interface.isEmpty() && !QDBusUtil::isValidInterfaceName(interface)) {
+#ifndef QT_NO_DEBUG
+        qWarning("QDBusConnection::connect: interface name '%s' is not valid", interface.toLatin1().constData());
+#endif
+        return false;
+    }
+    if (!service.isEmpty() && !QDBusUtil::isValidBusName(service)) {
+#ifndef QT_NO_DEBUG
+        qWarning("QDBusConnection::connect: service name '%s' is not valid", service.toLatin1().constData());
+#endif
+        return false;
+    }
+    if (!path.isEmpty() && !QDBusUtil::isValidObjectPath(path)) {
+#ifndef QT_NO_DEBUG
+        qWarning("QDBusConnection::connect: object path '%s' is not valid", path.toLatin1().constData());
+#endif
+        return false;
+    }
 
     QDBusWriteLocker locker(ConnectAction, d);
     return d->connectSignal(service, path, interface, name, argumentMatch, signature, receiver, slot);
@@ -729,9 +813,21 @@ bool QDBusConnection::registerObject(const QString &path, QObject *object, Regis
             // this node exists
             // consider it free if there's no object here and the user is not trying to
             // replace the object sub-tree
-            if ((options & ExportChildObjects && !node->children.isEmpty()) || node->obj)
+            if (node->obj)
                 return false;
 
+            if (options & QDBusConnectionPrivate::VirtualObject) {
+                // technically the check for children needs to go even deeper
+                if (options & SubPath) {
+                    foreach (const QDBusConnectionPrivate::ObjectTreeNode &child, node->children) {
+                        if (child.obj)
+                            return false;
+                    }
+                }
+            } else {
+                if ((options & ExportChildObjects && !node->children.isEmpty()))
+                    return false;
+            }
             // we can add the object here
             node->obj = object;
             node->flags = options;
@@ -741,6 +837,13 @@ bool QDBusConnection::registerObject(const QString &path, QObject *object, Regis
             return true;
         }
 
+        // if a virtual object occupies this path, return false
+        if (node->obj && (node->flags & QDBusConnectionPrivate::VirtualObject) && (node->flags & QDBusConnection::SubPath)) {
+            qDebug("Cannot register object at %s because QDBusVirtualObject handles all sub-paths.",
+                   qPrintable(path));
+            return false;
+        }
+
         // find the position where we'd insert the node
         QDBusConnectionPrivate::ObjectTreeNode::DataList::Iterator it =
             qLowerBound(node->children.begin(), node->children.end(), pathComponents.at(i));
@@ -768,6 +871,21 @@ bool QDBusConnection::registerObject(const QString &path, QObject *object, Regis
     return false;
 }
 
+/*!
+    \internal
+    \since 4.8
+    Registers a QDBusTreeNode for a path. It can handle a path including all child paths, thus
+    handling multiple DBus nodes.
+
+    To unregister a QDBusTreeNode use the unregisterObject() function with its path.
+*/
+bool QDBusConnection::registerVirtualObject(const QString &path, QDBusVirtualObject *treeNode,
+                      VirtualObjectRegisterOption options)
+{
+    int opts = options | QDBusConnectionPrivate::VirtualObject;
+    return registerObject(path, (QObject*) treeNode, (RegisterOptions) opts);
+}
+
 /*!
     Unregisters an object that was registered with the registerObject() at the object path given by
     \a path and, if \a mode is QDBusConnection::UnregisterTree, all of its sub-objects too.
@@ -786,7 +904,7 @@ void QDBusConnection::unregisterObject(const QString &path, UnregisterMode mode)
 
     // find the object
     while (node) {
-        if (pathComponents.count() == i) {
+        if (pathComponents.count() == i || !path.compare(QLatin1String("/"))) {
             // found it
             node->obj = 0;
             node->flags = 0;
@@ -819,7 +937,7 @@ QObject *QDBusConnection::objectRegisteredAt(const QString &path) const
     Q_ASSERT_X(QDBusUtil::isValidObjectPath(path), "QDBusConnection::registeredObject",
                "Invalid object path given");
     if (!d || !d->connection || !QDBusUtil::isValidObjectPath(path))
-        return false;
+        return 0;
 
     QStringList pathComponents = path.split(QLatin1Char('/'));
     if (pathComponents.last().isEmpty())
@@ -833,6 +951,8 @@ QObject *QDBusConnection::objectRegisteredAt(const QString &path) const
     while (node) {
         if (pathComponents.count() == i)
             return node->obj;
+        if ((node->flags & QDBusConnectionPrivate::VirtualObject) && (node->flags & QDBusConnection::SubPath))
+            return node->obj;
 
         QDBusConnectionPrivate::ObjectTreeNode::DataList::ConstIterator it =
             qLowerBound(node->children.constBegin(), node->children.constEnd(), pathComponents.at(i));
@@ -845,6 +965,8 @@ QObject *QDBusConnection::objectRegisteredAt(const QString &path) const
     return 0;
 }
 
+
+
 /*!
     Returns a QDBusConnectionInterface object that represents the
     D-Bus server interface on this connection.
@@ -856,6 +978,21 @@ QDBusConnectionInterface *QDBusConnection::interface() const
     return d->busService;
 }
 
+/*!
+    \internal
+    \since 4.8
+
+    Returns the internal, implementation-defined pointer for this
+    connection. Currently, this returns a DBusConnection* pointer,
+    without changing the reference count. It is the responsibility of
+    the caller to call dbus_connection_ref if it wants to store the
+    pointer.
+*/
+void *QDBusConnection::internalPointer() const
+{
+    return d ? d->connection : 0;
+}
+
 /*!
     Returns true if this QDBusConnection object is connected.
 */
@@ -915,6 +1052,18 @@ QString QDBusConnection::name() const
     return d ? d->name : QString();
 }
 
+/*!
+    \since 4.8
+
+    Returns the capabilities of this connection as negotiated with the bus
+    server or peer. If this QDBusConnection is not connected, this function
+    returns no capabilities.
+*/
+QDBusConnection::ConnectionCapabilities QDBusConnection::connectionCapabilities() const
+{
+    return d ? d->capabilities : ConnectionCapabilities(0);
+}
+
 /*!
     Attempts to register the \a serviceName on the D-Bus server and
     returns true if the registration succeeded. The registration will
@@ -956,7 +1105,16 @@ class QDBusDefaultConnection: public QDBusConnection
 public:
     inline QDBusDefaultConnection(BusType type, const char *name)
         : QDBusConnection(connectToBus(type, QString::fromLatin1(name))), ownName(name)
-    { }
+    {
+        // make sure this connection is running on the main thread
+        QCoreApplication *instance = QCoreApplication::instance();
+        if (!instance) {
+            qWarning("QDBusConnection: %s D-Bus connection created before QCoreApplication. Application may misbehave.",
+                     type == SessionBus ? "session" : type == SystemBus ? "system" : "generic");
+        } else if (QDBusConnectionPrivate::d(*this)) {
+            QDBusConnectionPrivate::d(*this)->moveToThread(instance->thread());
+        }
+    }
 
     inline ~QDBusDefaultConnection()
     { disconnectFromBus(QString::fromLatin1(ownName)); }
@@ -1015,6 +1173,27 @@ void QDBusConnectionPrivate::setBusService(const QDBusConnection &connection)
                      Qt::QueuedConnection);
 }
 
+/*!
+    \since 4.8
+    Returns the local machine ID as known to the D-Bus system. Each
+    node or host that runs D-Bus has a unique identifier that can be
+    used to distinguish it from other hosts if they are sharing
+    resources like the filesystem.
+
+    Note that the local machine ID is not guaranteed to be persistent
+    across boots of the system, so this identifier should not be
+    stored in persistent storage (like the filesystem). It is
+    guaranteed to remain constant only during the lifetime of this
+    boot session.
+*/
+QByteArray QDBusConnection::localMachineId()
+{
+    char *dbus_machine_id = q_dbus_get_local_machine_id();
+    QByteArray result = dbus_machine_id;
+    q_dbus_free(dbus_machine_id);
+    return result;
+}
+
 /*!
     \namespace QDBus
     \inmodule QtDBus
