@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -77,14 +77,33 @@
 
     \snippet doc/src/snippets/code/src_network_socket_qudpsocket.cpp 0
 
+    QUdpSocket also supports UDP multicast. Use joinMulticastGroup() and
+    leaveMulticastGroup() to control group membership, and
+    QAbstractSocket::MulticastTtlOption and
+    QAbstractSocket::MulticastLoopbackOption to set the TTL and loopback socket
+    options. Use setMulticastInterface() to control the outgoing interface for
+    multicast datagrams, and multicastInterface() to query it.
+
     With QUdpSocket, you can also establish a virtual connection to a
     UDP server using connectToHost() and then use read() and write()
     to exchange datagrams without specifying the receiver for each
     datagram.
 
-    The \l{network/broadcastsender}{Broadcast Sender} and
-    \l{network/broadcastreceiver}{Broadcast Receiver} examples
-    illustrate how to use QUdpSocket in applications.
+    The \l{network/broadcastsender}{Broadcast Sender},
+    \l{network/broadcastreceiver}{Broadcast Receiver},
+    \l{network/multicastsender}{Multicast Sender}, and
+    \l{network/multicastreceiver}{Multicast Receiver} examples illustrate how
+    to use QUdpSocket in applications.
+
+    \section1 Symbian Platform Security Requirements
+
+    On Symbian, processes which use this class must have the
+    \c NetworkServices platform security capability. If the client
+    process lacks this capability, operations will result in a panic.
+
+    Platform security capabilities are added via the
+    \l{qmake-variable-reference.html#target-capability}{TARGET.CAPABILITY}
+    qmake variable.
 
     \sa QTcpSocket
 */
@@ -135,6 +154,7 @@
 */
 
 #include "qhostaddress.h"
+#include "qnetworkinterface.h"
 #include "qabstractsocket_p.h"
 #include "qudpsocket.h"
 
@@ -182,7 +202,7 @@ bool QUdpSocketPrivate::doEnsureInitialized(const QHostAddress &bindAddress, qui
 #endif
 
     // now check if the socket engine is initialized and to the right type
-    if (!socketEngine || !socketEngine->isValid() || socketEngine->protocol() != proto) {
+    if (!socketEngine || !socketEngine->isValid()) {
         resolveProxy(remoteAddress.toString(), bindPort);
         if (!initSocketLayer(address->protocol()))
             return false;
@@ -318,6 +338,116 @@ bool QUdpSocket::bind(quint16 port, BindMode mode)
     return bind(QHostAddress::Any, port, mode);
 }
 
+#ifndef QT_NO_NETWORKINTERFACE
+
+/*!
+    \since 4.8
+
+    Joins the the multicast group specified by \a groupAddress on the default
+    interface chosen by the operating system. The socket must be in BoundState,
+    otherwise an error occurs.
+
+    This function returns true if successful; otherwise it returns false
+    and sets the socket error accordingly.
+
+    \sa leaveMulticastGroup()
+*/
+bool QUdpSocket::joinMulticastGroup(const QHostAddress &groupAddress)
+{
+    return joinMulticastGroup(groupAddress, QNetworkInterface());
+}
+
+/*!
+    \since 4.8
+    \overload
+
+    Joins the multicast group address \a groupAddress on the interface \a
+    iface.
+
+    \sa leaveMulticastGroup()
+*/
+bool QUdpSocket::joinMulticastGroup(const QHostAddress &groupAddress,
+                                    const QNetworkInterface &iface)
+{
+    Q_D(QUdpSocket);
+    QT_CHECK_BOUND("QUdpSocket::joinMulticastGroup()", false);
+    return d->socketEngine->joinMulticastGroup(groupAddress, iface);
+}
+
+/*!
+    \since 4.8
+
+    Leaves the multicast group specified by \a groupAddress on the default
+    interface chosen by the operating system. The socket must be in BoundState,
+    otherwise an error occurs.
+
+   This function returns true if successful; otherwise it returns false and
+   sets the socket error accordingly.
+
+   \sa joinMulticastGroup()
+*/
+bool QUdpSocket::leaveMulticastGroup(const QHostAddress &groupAddress)
+{
+    return leaveMulticastGroup(groupAddress, QNetworkInterface());
+}
+
+/*!
+    \since 4.8
+    \overload
+
+    Leaves the multicast group specified by \a groupAddress on the interface \a
+    iface.
+
+    \sa joinMulticastGroup()
+*/
+bool QUdpSocket::leaveMulticastGroup(const QHostAddress &groupAddress,
+                                     const QNetworkInterface &iface)
+{
+    QT_CHECK_BOUND("QUdpSocket::leaveMulticastGroup()", false);
+    return d_func()->socketEngine->leaveMulticastGroup(groupAddress, iface);
+}
+
+/*!
+    \since 4.8
+
+    Returns the interface for the outgoing interface for multicast datagrams.
+    This corresponds to the IP_MULTICAST_IF socket option for IPv4 sockets and
+    the IPV6_MULTICAST_IF socket option for IPv6 sockets. If no interface has
+    been previously set, this function returns an invalid QNetworkInterface.
+    The socket must be in BoundState, otherwise an invalid QNetworkInterface is
+    returned.
+
+    \sa setMulticastInterface()
+*/
+QNetworkInterface QUdpSocket::multicastInterface() const
+{
+    Q_D(const QUdpSocket);
+    QT_CHECK_BOUND("QUdpSocket::multicastInterface()", QNetworkInterface());
+    return d->socketEngine->multicastInterface();
+}
+
+/*!
+    \since 4.8
+
+    Sets the outgoing interface for multicast datagrams to the interface \a
+    iface. This corresponds to the IP_MULTICAST_IF socket option for IPv4
+    sockets and the IPV6_MULTICAST_IF socket option for IPv6 sockets. The
+    socket must be in BoundState, otherwise this function does nothing.
+
+    \sa multicastInterface(), joinMulticastGroup(), leaveMulticastGroup()
+*/
+void QUdpSocket::setMulticastInterface(const QNetworkInterface &iface)
+{
+    Q_D(QUdpSocket);
+    if (!isValid()) {
+        qWarning("QUdpSocket::setMulticastInterface() called on a QUdpSocket when not in QUdpSocket::BoundState");
+        return;
+    }
+    d->socketEngine->setMulticastInterface(iface);
+}
+
+#endif // QT_NO_NETWORKINTERFACE
+
 /*!
     Returns true if at least one datagram is waiting to be read;
     otherwise returns false.
@@ -378,16 +508,6 @@ qint64 QUdpSocket::writeDatagram(const char *data, qint64 size, const QHostAddre
         return -1;
 
     qint64 sent = d->socketEngine->writeDatagram(data, size, address, port);
-#ifdef Q_OS_SYMBIAN
-    if( QSysInfo::s60Version() <= QSysInfo::SV_S60_5_0 ) {
-        // This is evil hack, but for some reason native RSocket::SendTo returns 0,
-        // for large datagrams (such as 600 bytes). Based on comments from Open C team
-        // this should happen only in platforms <= S60 5.0.
-        // As an workaround, we just set sent = size
-        if( sent == 0 )
-            sent = size;
-    }
-#endif
     d->cachedSocketDescriptor = d->socketEngine->socketDescriptor();
 
     if (sent >= 0) {
