@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -216,8 +216,8 @@ Type Moc::parseType()
             QByteArray templ = lexemUntil(RANGLE);
             for (int i = 0; i < templ.size(); ++i) {
                 type.name += templ.at(i);
-                if ((templ.at(i) == '<' && i < templ.size()-1 && templ.at(i+1) == ':')
-                    || (templ.at(i) == '>' && i < templ.size()-1 && templ.at(i+1) == '>')) {
+                if ((templ.at(i) == '<' && i+1 < templ.size() && templ.at(i+1) == ':')
+                    || (templ.at(i) == '>' && i+1 < templ.size() && templ.at(i+1) == '>')) {
                     type.name += ' ';
                 }
             }
@@ -292,8 +292,8 @@ void Moc::parseFunctionArguments(FunctionDef *def)
             arg.rightType += ' ';
             arg.rightType += lexem();
         }
-        arg.normalizedType = normalizeType(arg.type.name + ' ' + arg.rightType);
-        arg.typeNameForCast = normalizeType(noRef(arg.type.name) + "(*)" + arg.rightType);
+        arg.normalizedType = normalizeType(QByteArray(arg.type.name + ' ' + arg.rightType));
+        arg.typeNameForCast = normalizeType(QByteArray(noRef(arg.type.name) + "(*)" + arg.rightType));
         if (test(EQ))
             arg.isDefault = true;
         def->arguments += arg;
@@ -335,14 +335,32 @@ bool Moc::testFunctionAttribute(Token tok, FunctionDef *def)
     return false;
 }
 
+bool Moc::testFunctionRevision(FunctionDef *def)
+{
+    if (test(Q_REVISION_TOKEN)) {
+        next(LPAREN);
+        QByteArray revision = lexemUntil(RPAREN);
+        revision.remove(0, 1);
+        revision.chop(1);
+        bool ok = false;
+        def->revision = revision.toInt(&ok);
+        if (!ok || def->revision < 0)
+            error("Invalid revision");
+        return true;
+    }
+
+    return false;
+}
+
 // returns false if the function should be ignored
 bool Moc::parseFunction(FunctionDef *def, bool inMacro)
 {
     def->isVirtual = false;
+    def->isStatic = false;
     //skip modifiers and attributes
-    while (test(INLINE) || test(STATIC) ||
+    while (test(INLINE) || (test(STATIC) && (def->isStatic = true)) ||
         (test(VIRTUAL) && (def->isVirtual = true)) //mark as virtual
-        || testFunctionAttribute(def)) {}
+        || testFunctionAttribute(def) || testFunctionRevision(def)) {}
     bool templateFunction = (lookup() == TEMPLATE);
     def->type = parseType();
     if (def->type.name.isEmpty()) {
@@ -430,10 +448,11 @@ bool Moc::parseFunction(FunctionDef *def, bool inMacro)
 bool Moc::parseMaybeFunction(const ClassDef *cdef, FunctionDef *def)
 {
     def->isVirtual = false;
+    def->isStatic = false;
     //skip modifiers and attributes
-    while (test(EXPLICIT) || test(INLINE) || test(STATIC) ||
+    while (test(EXPLICIT) || test(INLINE) || (test(STATIC) && (def->isStatic = true)) ||
         (test(VIRTUAL) && (def->isVirtual = true)) //mark as virtual
-        || testFunctionAttribute(def)) {}
+        || testFunctionAttribute(def) || testFunctionRevision(def)) {}
     bool tilde = test(TILDE);
     def->type = parseType();
     if (def->type.name.isEmpty())
@@ -668,10 +687,13 @@ void Moc::parse()
                     if (parseEnum(&enumDef))
                         def.enumList += enumDef;
                 } break;
+                case SEMIC:
+                case COLON:
+                    break;
                 default:
                     FunctionDef funcDef;
                     funcDef.access = access;
-                    int rewind = index;
+                    int rewind = index--;
                     if (parseMaybeFunction(&def, &funcDef)) {
                         if (funcDef.isConstructor) {
                             if ((access == FunctionDef::Public) && funcDef.isInvokable) {
@@ -694,6 +716,8 @@ void Moc::parse()
                                     funcDef.arguments.removeLast();
                                     def.slotList += funcDef;
                                 }
+                                if (funcDef.revision > 0)
+                                    ++def.revisionedMethods;
                             } else if (funcDef.isSignal) {
                                 def.signalList += funcDef;
                                 while (funcDef.arguments.size() > 0 && funcDef.arguments.last().isDefault) {
@@ -701,6 +725,8 @@ void Moc::parse()
                                     funcDef.arguments.removeLast();
                                     def.signalList += funcDef;
                                 }
+                                if (funcDef.revision > 0)
+                                    ++def.revisionedMethods;
                             } else if (funcDef.isInvokable) {
                                 def.methodList += funcDef;
                                 while (funcDef.arguments.size() > 0 && funcDef.arguments.last().isDefault) {
@@ -708,6 +734,8 @@ void Moc::parse()
                                     funcDef.arguments.removeLast();
                                     def.methodList += funcDef;
                                 }
+                                if (funcDef.revision > 0)
+                                    ++def.revisionedMethods;
                             }
                         }
                     } else {
@@ -738,9 +766,6 @@ void Moc::parse()
 
 void Moc::generate(FILE *out)
 {
-
-    QDateTime dt = QDateTime::currentDateTime();
-    QByteArray dstr = dt.toString().toLatin1();
     QByteArray fn = filename;
     int i = filename.length()-1;
     while (i>0 && filename[i-1] != '/' && filename[i-1] != '\\')
@@ -749,8 +774,7 @@ void Moc::generate(FILE *out)
         fn = filename.mid(i);
     fprintf(out, "/****************************************************************************\n"
             "** Meta object code from reading C++ file '%s'\n**\n" , (const char*)fn);
-    fprintf(out, "** Created: %s\n"
-            "**      by: The Qt Meta Object Compiler version %d (Qt %s)\n**\n" , dstr.data(), mocOutputRevision, QT_VERSION_STR);
+    fprintf(out, "** Created by: The Qt Meta Object Compiler version %d (Qt %s)\n**\n" , mocOutputRevision, QT_VERSION_STR);
     fprintf(out, "** WARNING! All changes made in this file will be lost!\n"
             "*****************************************************************************/\n\n");
 
@@ -806,6 +830,18 @@ QList<QMetaObject*> Moc::generate(bool ignoreProperties)
 
 void Moc::parseSlots(ClassDef *def, FunctionDef::Access access)
 {
+    int defaultRevision = -1;
+    if (test(Q_REVISION_TOKEN)) {
+        next(LPAREN);
+        QByteArray revision = lexemUntil(RPAREN);
+        revision.remove(0, 1);
+        revision.chop(1);
+        bool ok = false;
+        defaultRevision = revision.toInt(&ok);
+        if (!ok || defaultRevision < 0)
+            error("Invalid revision");
+    }
+
     next(COLON);
     while (inClass(def) && hasNext()) {
         switch (next()) {
@@ -831,6 +867,12 @@ void Moc::parseSlots(ClassDef *def, FunctionDef::Access access)
         funcDef.access = access;
         if (!parseFunction(&funcDef))
             continue;
+        if (funcDef.revision > 0) {
+            ++def->revisionedMethods;
+        } else if (defaultRevision != -1) {
+            funcDef.revision = defaultRevision;
+            ++def->revisionedMethods;
+        }
         def->slotList += funcDef;
         while (funcDef.arguments.size() > 0 && funcDef.arguments.last().isDefault) {
             funcDef.wasCloned = true;
@@ -842,6 +884,18 @@ void Moc::parseSlots(ClassDef *def, FunctionDef::Access access)
 
 void Moc::parseSignals(ClassDef *def)
 {
+    int defaultRevision = -1;
+    if (test(Q_REVISION_TOKEN)) {
+        next(LPAREN);
+        QByteArray revision = lexemUntil(RPAREN);
+        revision.remove(0, 1);
+        revision.chop(1);
+        bool ok = false;
+        defaultRevision = revision.toInt(&ok);
+        if (!ok || defaultRevision < 0)
+            error("Invalid revision");
+    }
+
     next(COLON);
     while (inClass(def) && hasNext()) {
         switch (next()) {
@@ -869,6 +923,12 @@ void Moc::parseSignals(ClassDef *def)
             warning("Signals cannot be declared virtual");
         if (funcDef.inlineCode)
             error("Not a signal declaration");
+        if (funcDef.revision > 0) {
+            ++def->revisionedMethods;
+        } else if (defaultRevision != -1) {
+            funcDef.revision = defaultRevision;
+            ++def->revisionedMethods;
+        }
         def->signalList += funcDef;
         while (funcDef.arguments.size() > 0 && funcDef.arguments.last().isDefault) {
             funcDef.wasCloned = true;
@@ -911,7 +971,6 @@ void Moc::createPropertyDef(PropertyDef &propDef)
     propDef.name = lexem();
     while (test(IDENTIFIER)) {
         QByteArray l = lexem();
-
         if (l[0] == 'C' && l == "CONSTANT") {
             propDef.constant = true;
             continue;
@@ -923,6 +982,10 @@ void Moc::createPropertyDef(PropertyDef &propDef)
         QByteArray v, v2;
         if (test(LPAREN)) {
             v = lexemUntil(RPAREN);
+        } else if (test(INTEGER_LITERAL)) {
+            v = lexem();
+            if (l != "REVISION")
+                error(1);
         } else {
             next(IDENTIFIER);
             v = lexem();
@@ -937,7 +1000,12 @@ void Moc::createPropertyDef(PropertyDef &propDef)
                 propDef.read = v;
             else if (l == "RESET")
                 propDef.reset = v + v2;
-            else
+            else if (l == "REVISION") {
+                bool ok = false;
+                propDef.revision = v.toInt(&ok);
+                if (!ok || propDef.revision < 0)
+                    error(1);
+            } else
                 error(2);
             break;
         case 'S':
@@ -1002,6 +1070,8 @@ void Moc::parseProperty(ClassDef *def)
 
     if(!propDef.notify.isEmpty())
         def->notifyableProperties++;
+    if (propDef.revision > 0)
+        ++def->revisionedProperties;
     def->propertyList += propDef;
 }
 
@@ -1028,6 +1098,8 @@ void Moc::parsePrivateProperty(ClassDef *def)
 
     if(!propDef.notify.isEmpty())
         def->notifyableProperties++;
+    if (propDef.revision > 0)
+        ++def->revisionedProperties;
 
     def->propertyList += propDef;
 }
@@ -1177,6 +1249,9 @@ void Moc::parseSlotInPrivate(ClassDef *def, FunctionDef::Access access)
         funcDef.arguments.removeLast();
         def->slotList += funcDef;
     }
+    if (funcDef.revision > 0)
+        ++def->revisionedMethods;
+
 }
 
 QByteArray Moc::lexemUntil(Token target)
