@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the Qt Linguist of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -42,6 +42,7 @@
 #include "lupdate.h"
 
 #include <translator.h>
+#include <profileparser.h>
 #include <profileevaluator.h>
 
 #include <QtCore/QCoreApplication>
@@ -61,7 +62,14 @@ static QString m_defaultExtensions;
 
 static void printOut(const QString & out)
 {
-    std::cerr << qPrintable(out);
+    QTextStream stream(stdout);
+    stream << out;
+}
+
+static void printErr(const QString & out)
+{
+    QTextStream stream(stderr);
+    stream << out;
 }
 
 class LU {
@@ -151,23 +159,23 @@ static void updateTsFiles(const Translator &fetchedTor, const QStringList &tsFil
         cd.m_sortContexts = !(options & NoSort);
         if (QFile(fileName).exists()) {
             if (!tor.load(fileName, cd, QLatin1String("auto"))) {
-                printOut(cd.error());
+                printErr(cd.error());
                 *fail = true;
                 continue;
             }
             tor.resolveDuplicates();
             cd.clearErrors();
             if (setCodec && fetchedTor.codec() != tor.codec())
-                printOut(LU::tr("lupdate warning: Codec for tr() '%1' disagrees with"
+                printErr(LU::tr("lupdate warning: Codec for tr() '%1' disagrees with"
                                 " existing file's codec '%2'. Expect trouble.\n")
                          .arg(QString::fromLatin1(fetchedTor.codecName()),
                               QString::fromLatin1(tor.codecName())));
             if (!targetLanguage.isEmpty() && targetLanguage != tor.languageCode())
-                printOut(LU::tr("lupdate warning: Specified target language '%1' disagrees with"
+                printErr(LU::tr("lupdate warning: Specified target language '%1' disagrees with"
                                 " existing file's language '%2'. Ignoring.\n")
                          .arg(targetLanguage, tor.languageCode()));
             if (!sourceLanguage.isEmpty() && sourceLanguage != tor.sourceLanguageCode())
-                printOut(LU::tr("lupdate warning: Specified source language '%1' disagrees with"
+                printErr(LU::tr("lupdate warning: Specified source language '%1' disagrees with"
                                 " existing file's language '%2'. Ignoring.\n")
                          .arg(sourceLanguage, tor.sourceLanguageCode()));
         } else {
@@ -212,16 +220,50 @@ static void updateTsFiles(const Translator &fetchedTor, const QStringList &tsFil
 
         out.normalizeTranslations(cd);
         if (!cd.errors().isEmpty()) {
-            printOut(cd.error());
+            printErr(cd.error());
             cd.clearErrors();
         }
         if (!out.save(fileName, cd, QLatin1String("auto"))) {
-            printOut(cd.error());
+            printErr(cd.error());
             *fail = true;
         }
     }
 }
 
+static void print(const QString &fileName, int lineNo, const QString &msg)
+{
+    if (lineNo)
+        printErr(QString::fromLatin1("%2(%1): %3").arg(lineNo).arg(fileName, msg));
+    else
+        printErr(msg);
+}
+
+class ParseHandler : public ProFileParserHandler {
+public:
+    virtual void parseError(const QString &fileName, int lineNo, const QString &msg)
+        { if (verbose) print(fileName, lineNo, msg); }
+
+    bool verbose;
+};
+
+class EvalHandler : public ProFileEvaluatorHandler {
+public:
+    virtual void configError(const QString &msg)
+        { printErr(msg); }
+    virtual void evalError(const QString &fileName, int lineNo, const QString &msg)
+        { if (verbose) print(fileName, lineNo, msg); }
+    virtual void fileMessage(const QString &msg)
+        { printErr(msg); }
+
+    virtual void aboutToEval(ProFile *, ProFile *, EvalFileType) {}
+    virtual void doneWithEval(ProFile *) {}
+
+    bool verbose;
+};
+
+static ParseHandler parseHandler;
+static EvalHandler evalHandler;
+
 static QStringList getSources(const char *var, const char *vvar, const QStringList &baseVPaths,
                               const QString &projectDir, const ProFileEvaluator &visitor)
 {
@@ -278,17 +320,19 @@ static void processSources(Translator &fetchedTor,
     }
     loadCPP(fetchedTor, sourceFilesCpp, cd);
     if (!cd.error().isEmpty())
-        printOut(cd.error());
+        printErr(cd.error());
 }
 
 static void processProjects(
         bool topLevel, bool nestComplain, const QStringList &proFiles,
+        ProFileOption *option, ProFileParser *parser,
         UpdateOptions options, const QByteArray &codecForSource,
         const QString &targetLanguage, const QString &sourceLanguage,
         Translator *parentTor, bool *fail);
 
 static void processProject(
-        bool nestComplain, const QFileInfo &pfi, ProFileEvaluator &visitor,
+        bool nestComplain, const QFileInfo &pfi,
+        ProFileOption *option, ProFileParser *parser, ProFileEvaluator &visitor,
         UpdateOptions options, const QByteArray &_codecForSource,
         const QString &targetLanguage, const QString &sourceLanguage,
         Translator *fetchedTor, bool *fail)
@@ -298,7 +342,7 @@ static void processProject(
     if (!tmp.isEmpty()) {
         codecForSource = tmp.last().toLatin1();
         if (!QTextCodec::codecForName(codecForSource)) {
-            printOut(LU::tr("lupdate warning: Codec for source '%1' is invalid."
+            printErr(LU::tr("lupdate warning: Codec for source '%1' is invalid."
                             " Falling back to codec for tr().\n")
                      .arg(QString::fromLatin1(codecForSource)));
             codecForSource.clear();
@@ -308,7 +352,12 @@ static void processProject(
         QStringList subProFiles;
         QDir proDir(pfi.absoluteDir());
         foreach (const QString &subdir, visitor.values(QLatin1String("SUBDIRS"))) {
-            QString subPro = QDir::cleanPath(proDir.absoluteFilePath(subdir));
+            QString realdir = visitor.value(subdir + QLatin1String(".subdir"));
+            if (realdir.isEmpty())
+                realdir = visitor.value(subdir + QLatin1String(".file"));
+            if (realdir.isEmpty())
+                realdir = subdir;
+            QString subPro = QDir::cleanPath(proDir.absoluteFilePath(realdir));
             QFileInfo subInfo(subPro);
             if (subInfo.isDir())
                 subProFiles << (subPro + QLatin1Char('/')
@@ -316,7 +365,7 @@ static void processProject(
             else
                 subProFiles << subPro;
         }
-        processProjects(false, nestComplain, subProFiles, options, codecForSource,
+        processProjects(false, nestComplain, subProFiles, option, parser, options, codecForSource,
                         targetLanguage, sourceLanguage, fetchedTor, fail);
     } else {
         ConversionData cd;
@@ -342,31 +391,38 @@ static void processProject(
 
 static void processProjects(
         bool topLevel, bool nestComplain, const QStringList &proFiles,
+        ProFileOption *option, ProFileParser *parser,
         UpdateOptions options, const QByteArray &codecForSource,
         const QString &targetLanguage, const QString &sourceLanguage,
         Translator *parentTor, bool *fail)
 {
     foreach (const QString &proFile, proFiles) {
-        ProFileEvaluator visitor;
-        visitor.setVerbose(options & Verbose);
-
         QFileInfo pfi(proFile);
-        ProFile pro(pfi.absoluteFilePath());
-        if (!visitor.queryProFile(&pro) || !visitor.accept(&pro)) {
+
+        ProFileEvaluator visitor(option, parser, &evalHandler);
+        ProFile *pro;
+        if (!(pro = parser->parsedProFile(QDir::cleanPath(pfi.absoluteFilePath())))) {
             if (topLevel)
                 *fail = true;
             continue;
         }
+        if (!visitor.accept(pro)) {
+            if (topLevel)
+                *fail = true;
+            pro->deref();
+            continue;
+        }
 
         if (visitor.contains(QLatin1String("TRANSLATIONS"))) {
             if (parentTor) {
                 if (topLevel) {
-                    std::cerr << qPrintable(LU::tr("lupdate warning: TS files from command line "
-                                                   "will override TRANSLATIONS in %1.\n").arg(proFile));
+                    printErr(LU::tr("lupdate warning: TS files from command line "
+                                    "will override TRANSLATIONS in %1.\n").arg(proFile));
                     goto noTrans;
                 } else if (nestComplain) {
-                    std::cerr << qPrintable(LU::tr("lupdate warning: TS files from command line "
-                                                   "prevent recursing into %1.\n").arg(proFile));
+                    printErr(LU::tr("lupdate warning: TS files from command line "
+                                    "prevent recursing into %1.\n").arg(proFile));
+                    pro->deref();
                     continue;
                 }
             }
@@ -378,6 +434,7 @@ static void processProjects(
                 // This might mean either a buggy PRO file or an intentional detach -
                 // we can't know without seeing the actual RHS of the assignment ...
                 // Just assume correctness and be silent.
+                pro->deref();
                 continue;
             }
             Translator tor;
@@ -389,29 +446,32 @@ static void processProjects(
                 tor.setCodecName(tmp.last().toLatin1());
                 setCodec = true;
             }
-            processProject(false, pfi, visitor, options, codecForSource,
+            processProject(false, pfi, option, parser, visitor, options, codecForSource,
                            targetLanguage, sourceLanguage, &tor, fail);
             updateTsFiles(tor, tsFiles, setCodec, sourceLanguage, targetLanguage, options, fail);
+            pro->deref();
             continue;
         }
       noTrans:
         if (!parentTor) {
             if (topLevel)
-                std::cerr << qPrintable(LU::tr("lupdate warning: no TS files specified. Only diagnostics "
-                                               "will be produced for '%1'.\n").arg(proFile));
+                printErr(LU::tr("lupdate warning: no TS files specified. Only diagnostics "
+                                "will be produced for '%1'.\n").arg(proFile));
             Translator tor;
-            processProject(nestComplain, pfi, visitor, options, codecForSource,
+            processProject(nestComplain, pfi, option, parser, visitor, options, codecForSource,
                            targetLanguage, sourceLanguage, &tor, fail);
         } else {
-            processProject(nestComplain, pfi, visitor, options, codecForSource,
+            processProject(nestComplain, pfi, option, parser, visitor, options, codecForSource,
                            targetLanguage, sourceLanguage, parentTor, fail);
         }
+        pro->deref();
     }
 }
 
 int main(int argc, char **argv)
 {
     QCoreApplication app(argc, argv);
+#ifndef Q_OS_WIN32
     QTranslator translator;
     QTranslator qtTranslator;
     QString sysLocale = QLocale::system().name();
@@ -421,6 +481,7 @@ int main(int argc, char **argv)
         app.installTranslator(&translator);
         app.installTranslator(&qtTranslator);
     }
+#endif // Q_OS_WIN32
 
     m_defaultExtensions = QLatin1String("java,jui,ui,c,c++,cc,cpp,cxx,ch,h,h++,hh,hpp,hxx,js,qs,qml");
 
@@ -452,6 +513,9 @@ int main(int argc, char **argv)
                 || arg == QLatin1String("-h")) {
             printUsage();
             return 0;
+        } else if (arg == QLatin1String("-list-languages")) {
+            printOut(getNumerusInfoString());
+            return 0;
         } else if (arg == QLatin1String("-pluralonly")) {
             options |= PluralOnly;
             continue;
@@ -465,7 +529,7 @@ int main(int argc, char **argv)
         } else if (arg == QLatin1String("-target-language")) {
             ++i;
             if (i == argc) {
-                printOut(LU::tr("The option -target-language requires a parameter.\n"));
+                printErr(LU::tr("The option -target-language requires a parameter.\n"));
                 return 1;
             }
             targetLanguage = args[i];
@@ -473,7 +537,7 @@ int main(int argc, char **argv)
         } else if (arg == QLatin1String("-source-language")) {
             ++i;
             if (i == argc) {
-                printOut(LU::tr("The option -source-language requires a parameter.\n"));
+                printErr(LU::tr("The option -source-language requires a parameter.\n"));
                 return 1;
             }
             sourceLanguage = args[i];
@@ -481,7 +545,7 @@ int main(int argc, char **argv)
         } else if (arg == QLatin1String("-disable-heuristic")) {
             ++i;
             if (i == argc) {
-                printOut(LU::tr("The option -disable-heuristic requires a parameter.\n"));
+                printErr(LU::tr("The option -disable-heuristic requires a parameter.\n"));
                 return 1;
             }
             arg = args[i];
@@ -492,14 +556,14 @@ int main(int argc, char **argv)
             } else if (arg == QLatin1String("number")) {
                 options &= ~HeuristicNumber;
             } else {
-                printOut(LU::tr("Invalid heuristic name passed to -disable-heuristic.\n"));
+                printErr(LU::tr("Invalid heuristic name passed to -disable-heuristic.\n"));
                 return 1;
             }
             continue;
         } else if (arg == QLatin1String("-locations")) {
             ++i;
             if (i == argc) {
-                printOut(LU::tr("The option -locations requires a parameter.\n"));
+                printErr(LU::tr("The option -locations requires a parameter.\n"));
                 return 1;
             }
             if (args[i] == QLatin1String("none")) {
@@ -509,7 +573,7 @@ int main(int argc, char **argv)
             } else if (args[i] == QLatin1String("absolute")) {
                 options |= AbsoluteLocations;
             } else {
-                printOut(LU::tr("Invalid parameter passed to -locations.\n"));
+                printErr(LU::tr("Invalid parameter passed to -locations.\n"));
                 return 1;
             }
             continue;
@@ -535,7 +599,7 @@ int main(int argc, char **argv)
         } else if (arg == QLatin1String("-codecfortr")) {
             ++i;
             if (i == argc) {
-                printOut(LU::tr("The -codecfortr option should be followed by a codec name.\n"));
+                printErr(LU::tr("The -codecfortr option should be followed by a codec name.\n"));
                 return 1;
             }
             codecForTr = args[i].toLatin1();
@@ -546,7 +610,7 @@ int main(int argc, char **argv)
         } else if (arg == QLatin1String("-extensions")) {
             ++i;
             if (i == argc) {
-                printOut(LU::tr("The -extensions option should be followed by an extension list.\n"));
+                printErr(LU::tr("The -extensions option should be followed by an extension list.\n"));
                 return 1;
             }
             extensions = args[i];
@@ -554,7 +618,7 @@ int main(int argc, char **argv)
         } else if (arg == QLatin1String("-pro")) {
             ++i;
             if (i == argc) {
-                printOut(LU::tr("The -pro option should be followed by a filename of .pro file.\n"));
+                printErr(LU::tr("The -pro option should be followed by a filename of .pro file.\n"));
                 return 1;
             }
             proFiles += args[i];
@@ -564,7 +628,7 @@ int main(int argc, char **argv)
             if (arg.length() == 2) {
                 ++i;
                 if (i == argc) {
-                    printOut(LU::tr("The -I option should be followed by a path.\n"));
+                    printErr(LU::tr("The -I option should be followed by a path.\n"));
                     return 1;
                 }
                 includePath += args[i];
@@ -573,7 +637,7 @@ int main(int argc, char **argv)
             }
             continue;
         } else if (arg.startsWith(QLatin1String("-")) && arg != QLatin1String("-")) {
-            printOut(LU::tr("Unrecognized option '%1'.\n").arg(arg));
+            printErr(LU::tr("Unrecognized option '%1'.\n").arg(arg));
             return 1;
         }
 
@@ -581,7 +645,7 @@ int main(int argc, char **argv)
         if (arg.startsWith(QLatin1String("@"))) {
             QFile lstFile(arg.mid(1));
             if (!lstFile.open(QIODevice::ReadOnly)) {
-                printOut(LU::tr("lupdate error: List file '%1' is not readable.\n")
+                printErr(LU::tr("lupdate error: List file '%1' is not readable.\n")
                          .arg(lstFile.fileName()));
                 return 1;
             }
@@ -599,7 +663,7 @@ int main(int argc, char **argv)
                         if (!fi.exists() || fi.isWritable()) {
                             tsFileNames.append(QFileInfo(file).absoluteFilePath());
                         } else {
-                            printOut(LU::tr("lupdate warning: For some reason, '%1' is not writable.\n")
+                            printErr(LU::tr("lupdate warning: For some reason, '%1' is not writable.\n")
                                      .arg(file));
                         }
                         found = true;
@@ -607,7 +671,7 @@ int main(int argc, char **argv)
                     }
                 }
                 if (!found) {
-                    printOut(LU::tr("lupdate error: File '%1' has no recognized extension.\n")
+                    printErr(LU::tr("lupdate error: File '%1' has no recognized extension.\n")
                              .arg(file));
                     return 1;
                 }
@@ -617,7 +681,7 @@ int main(int argc, char **argv)
             foreach (const QString &file, files) {
                 QFileInfo fi(file);
                 if (!fi.exists()) {
-                    printOut(LU::tr("lupdate error: File '%1' does not exist.\n").arg(file));
+                    printErr(LU::tr("lupdate error: File '%1' does not exist.\n").arg(file));
                     return 1;
                 }
                 if (file.endsWith(QLatin1String(".pro"), Qt::CaseInsensitive)
@@ -676,15 +740,15 @@ int main(int argc, char **argv)
     }
 
     if (!targetLanguage.isEmpty() && tsFileNames.count() != 1)
-        printOut(LU::tr("lupdate warning: -target-language usually only"
+        printErr(LU::tr("lupdate warning: -target-language usually only"
                         " makes sense with exactly one TS file.\n"));
     if (!codecForTr.isEmpty() && tsFileNames.isEmpty())
-        printOut(LU::tr("lupdate warning: -codecfortr has no effect without -ts.\n"));
+        printErr(LU::tr("lupdate warning: -codecfortr has no effect without -ts.\n"));
 
     bool fail = false;
     if (proFiles.isEmpty()) {
         if (tsFileNames.isEmpty())
-            printOut(LU::tr("lupdate warning:"
+            printErr(LU::tr("lupdate warning:"
                             " no TS files specified. Only diagnostics will be produced.\n"));
 
         Translator fetchedTor;
@@ -699,19 +763,26 @@ int main(int argc, char **argv)
                       sourceLanguage, targetLanguage, options, &fail);
     } else {
         if (!sourceFiles.isEmpty() || !includePath.isEmpty()) {
-            printOut(LU::tr("lupdate error:"
+            printErr(LU::tr("lupdate error:"
                             " Both project and source files / include paths specified.\n"));
             return 1;
         }
+
+        parseHandler.verbose = evalHandler.verbose = !!(options & Verbose);
+        ProFileOption option;
+        option.initProperties(app.applicationDirPath() + QLatin1String("/qmake"));
+        option.setCommandLineArguments(QStringList() << QLatin1String("CONFIG+=lupdate_run"));
+        ProFileParser parser(0, &parseHandler);
+
         if (!tsFileNames.isEmpty()) {
             Translator fetchedTor;
             fetchedTor.setCodecName(codecForTr);
-            processProjects(true, true, proFiles, options, QByteArray(),
+            processProjects(true, true, proFiles, &option, &parser, options, QByteArray(),
                             targetLanguage, sourceLanguage, &fetchedTor, &fail);
             updateTsFiles(fetchedTor, tsFileNames, !codecForTr.isEmpty(),
                           sourceLanguage, targetLanguage, options, &fail);
         } else {
-            processProjects(true, false, proFiles, options, QByteArray(),
+            processProjects(true, false, proFiles, &option, &parser, options, QByteArray(),
                             targetLanguage, sourceLanguage, 0, &fail);
         }
     }
