@@ -1,17 +1,18 @@
 /****************************************************************************
 **
-** Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies).
-** All rights reserved.
-** Contact: Nokia Corporation (qt-info@nokia.com)
+** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
-** Commercial Usage
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
+** a written agreement between you and Digia.  For licensing terms and
+** conditions see http://qt.digia.com/licensing.  For further information
+** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
@@ -21,8 +22,8 @@
 ** ensure the GNU Lesser General Public License version 2.1 requirements
 ** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** In addition, as a special exception, Digia gives you certain additional
+** rights.  These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -33,8 +34,7 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-** If you have questions regarding the use of this file, please contact
-** Nokia at qt-info@nokia.com.
+**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -247,7 +247,7 @@ bool ActiveSyncConnection::copyFileFromDevice(const QString &deviceSource, const
     wprintf(L"\n");
 
     if (!readUntilEnd) {
-        debugOutput(QString::fromLatin1("  an error occured during copy"), 2);
+        debugOutput(QString::fromLatin1("  an error occurred during copy"), 2);
         return false;
     }
 
@@ -443,6 +443,145 @@ bool ActiveSyncConnection::execute(QString program, QString arguments, int timeo
     return result;
 }
 
+bool ActiveSyncConnection::setDeviceAwake(bool activate, int *returnValue)
+{
+    if (!isConnected()) {
+        qWarning("Cannot execute, connect to device first!");
+        return false;
+    }
+    bool result = false;
+
+    // If we want to wait, we have to use CeRapiInvoke, as CeCreateProcess has no way to wait
+    // until the process ends. The lib must have been build and also deployed already.
+    if (!isConnected() && !connect())
+        return false;
+
+    HRESULT res = S_OK;
+
+    //SYSTEM_POWER_STATUS_EX systemPowerState;
+
+    //res = CeGetSystemPowerStatusEx(&systemPowerState, true);
+
+    QString dllLocation = "\\Windows\\QtRemote.dll";
+    QString functionName = "qRemoteToggleUnattendedPowerMode";
+
+    DWORD outputSize;
+    BYTE* output;
+    IRAPIStream *stream;
+    int returned = 0;
+    int toggle = int(activate);
+
+    res = CeRapiInvoke(dllLocation.utf16(), functionName.utf16(), 0, 0, &outputSize, &output, &stream, 0);
+    if (S_OK != res) {
+        DWORD ce_error = CeGetLastError();
+        if (S_OK != ce_error) {
+            qWarning("Error invoking %s on %s: %s", qPrintable(functionName),
+                qPrintable(dllLocation), strwinerror(ce_error).constData());
+        } else {
+            qWarning("Error: %s on %s unexpectedly returned %d", qPrintable(functionName),
+                qPrintable(dllLocation), res);
+        }
+    } else {
+        DWORD written;
+
+        if (S_OK != stream->Write(&toggle, sizeof(toggle), &written)) {
+            qWarning("   Could not write toggle option to process");
+            return false;
+        }
+
+        if (S_OK != stream->Read(&returned, sizeof(returned), &written)) {
+            qWarning("   Could not access return value of process");
+        }
+        else
+            result = true;
+    }
+
+    if (returnValue)
+        *returnValue = returned;
+
+    return result;
+}
+
+bool ActiveSyncConnection::resetDevice()
+{
+    if (!isConnected()) {
+        qWarning("Cannot execute, connect to device first!");
+        return false;
+    }
+
+    bool result = false;
+    if (!isConnected() && !connect())
+        return false;
+
+    QString dllLocation = "\\Windows\\QtRemote.dll";
+    QString functionName = "qRemoteSoftReset";
+
+    DWORD outputSize;
+    BYTE* output;
+    IRAPIStream *stream;
+
+    int returned = 0;
+
+    HRESULT res = CeRapiInvoke(dllLocation.utf16(), functionName.utf16(), 0, 0, &outputSize, &output, &stream, 0);
+    if (S_OK != res) {
+        DWORD ce_error = CeGetLastError();
+        if (S_OK != ce_error) {
+            qWarning("Error invoking %s on %s: %s", qPrintable(functionName),
+                qPrintable(dllLocation), strwinerror(ce_error).constData());
+        } else {
+            qWarning("Error: %s on %s unexpectedly returned %d", qPrintable(functionName),
+                qPrintable(dllLocation), res);
+        }
+    } else {
+        result = true;
+    }
+    return result;
+}
+
+bool ActiveSyncConnection::toggleDevicePower(int *returnValue)
+{
+    if (!isConnected()) {
+        qWarning("Cannot execute, connect to device first!");
+        return false;
+    }
+
+    bool result = false;
+    if (!isConnected() && !connect())
+        return false;
+
+    QString dllLocation = "\\Windows\\QtRemote.dll";
+    QString functionName = "qRemotePowerButton";
+
+    DWORD outputSize;
+    BYTE* output;
+    IRAPIStream *stream;
+    int returned = 0;
+
+    HRESULT res = CeRapiInvoke(dllLocation.utf16(), functionName.utf16(), 0, 0, &outputSize, &output, &stream, 0);
+    if (S_OK != res) {
+        DWORD ce_error = CeGetLastError();
+        if (S_OK != ce_error) {
+            qWarning("Error invoking %s on %s: %s", qPrintable(functionName),
+                qPrintable(dllLocation), strwinerror(ce_error).constData());
+        } else {
+            qWarning("Error: %s on %s unexpectedly returned %d", qPrintable(functionName),
+                qPrintable(dllLocation), res);
+        }
+    } else {
+        DWORD written;
+        if (S_OK != stream->Read(&returned, sizeof(returned), &written)) {
+            qWarning("   Could not access return value of process");
+        }
+        else {
+            result = true;
+        }
+    }
+
+    if (returnValue)
+        *returnValue = returned;
+    return result;
+}
+
 bool ActiveSyncConnection::createDirectory(const QString &path, bool deleteBefore)
 {
     if (deleteBefore)
